@@ -1,20 +1,25 @@
 # -*- Mode: Python; py-indent-offset: 4 -*-
+# coding=utf-8
 # vim: tabstop=4 shiftwidth=4 expandtab
 
-import pygtk
-pygtk.require("2.0")
+import sys
 
 import unittest
 from gi.repository import GObject
 
-import sys
-sys.path.insert(0, "../")
-
 import gobject
 from gi.repository import GIMarshallingTests
 
+from compathelper import _bytes
+
+if sys.version_info < (3, 0):
+    CONSTANT_UTF8 = "const \xe2\x99\xa5 utf8"
+    PY2_UNICODE_UTF8 = unicode(CONSTANT_UTF8, 'UTF-8')
+    CHAR_255='\xff'
+else:
+    CONSTANT_UTF8 = "const ♥ utf8"
+    CHAR_255=bytes([255])
 
-CONSTANT_UTF8 = "const \xe2\x99\xa5 utf8"
 CONSTANT_NUMBER = 42
 
 
@@ -118,9 +123,9 @@ class TestUInt8(unittest.TestCase):
         number = Number(self.MAX)
 
         GIMarshallingTests.uint8_in(number)
+        GIMarshallingTests.uint8_in(CHAR_255)
 
         number.value += 1
-
         self.assertRaises(ValueError, GIMarshallingTests.uint8_in, number)
         self.assertRaises(ValueError, GIMarshallingTests.uint8_in, Number(-1))
 
@@ -400,6 +405,7 @@ class TestInt(unittest.TestCase):
     def test_int_inout(self):
         self.assertEquals(self.MIN, GIMarshallingTests.int_inout_max_min(Number(self.MAX)))
         self.assertEquals(self.MAX, GIMarshallingTests.int_inout_min_max(Number(self.MIN)))
+        self.assertRaises(TypeError, GIMarshallingTests.int_inout_min_max, Number(self.MIN), CONSTANT_NUMBER)
 
 
 class TestUInt(unittest.TestCase):
@@ -613,13 +619,12 @@ class TestUtf8(unittest.TestCase):
 
     def test_utf8_none_in(self):
         GIMarshallingTests.utf8_none_in(CONSTANT_UTF8)
+        if sys.version_info < (3, 0):
+            GIMarshallingTests.utf8_none_in(PY2_UNICODE_UTF8)
 
         self.assertRaises(TypeError, GIMarshallingTests.utf8_none_in, CONSTANT_NUMBER)
         self.assertRaises(TypeError, GIMarshallingTests.utf8_none_in, None)
 
-    def test_utf8_full_in(self):
-        GIMarshallingTests.utf8_full_in(CONSTANT_UTF8)
-
     def test_utf8_none_out(self):
         self.assertEquals(CONSTANT_UTF8, GIMarshallingTests.utf8_none_out())
 
@@ -668,6 +673,10 @@ class TestArray(unittest.TestCase):
     def test_array_in(self):
         GIMarshallingTests.array_in(Sequence([-1, 0, 1, 2]))
 
+    def test_array_uint8_in(self):
+        GIMarshallingTests.array_uint8_in(Sequence([97, 98, 99, 100]))
+        GIMarshallingTests.array_uint8_in(_bytes("abcd"))
+
     def test_array_out(self):
         self.assertEquals([-1, 0, 1, 2], GIMarshallingTests.array_out())
 
@@ -754,12 +763,6 @@ class TestGArray(unittest.TestCase):
     def test_garray_utf8_none_in(self):
         GIMarshallingTests.garray_utf8_none_in(Sequence(['0', '1', '2']))
 
-    def test_garray_utf8_container_in(self):
-        GIMarshallingTests.garray_utf8_container_in(Sequence(['0', '1', '2']))
-
-    def test_garray_utf8_full_in(self):
-        GIMarshallingTests.garray_utf8_full_in(Sequence(['0', '1', '2']))
-
     def test_garray_utf8_none_out(self):
         self.assertEquals(['0', '1', '2'], GIMarshallingTests.garray_utf8_none_out())
 
@@ -804,12 +807,6 @@ class TestGList(unittest.TestCase):
     def test_glist_utf8_none_in(self):
         GIMarshallingTests.glist_utf8_none_in(Sequence(('0', '1', '2')))
 
-    def test_glist_utf8_container_in(self):
-        GIMarshallingTests.glist_utf8_container_in(Sequence(('0', '1', '2')))
-
-    def test_glist_utf8_full_in(self):
-        GIMarshallingTests.glist_utf8_full_in(Sequence(('0', '1', '2')))
-
     def test_glist_utf8_none_out(self):
         self.assertEquals(['0', '1', '2'], GIMarshallingTests.glist_utf8_none_out())
 
@@ -854,12 +851,6 @@ class TestGSList(unittest.TestCase):
     def test_gslist_utf8_none_in(self):
         GIMarshallingTests.gslist_utf8_none_in(Sequence(('0', '1', '2')))
 
-    def test_gslist_utf8_container_in(self):
-        GIMarshallingTests.gslist_utf8_container_in(Sequence(('0', '1', '2')))
-
-    def test_gslist_utf8_full_in(self):
-        GIMarshallingTests.gslist_utf8_full_in(Sequence(('0', '1', '2')))
-
     def test_gslist_utf8_none_out(self):
         self.assertEquals(['0', '1', '2'], GIMarshallingTests.gslist_utf8_none_out())
 
@@ -905,12 +896,6 @@ class TestGHashTable(unittest.TestCase):
     def test_ghashtable_utf8_none_in(self):
         GIMarshallingTests.ghashtable_utf8_none_in({'-1': '1', '0': '0', '1': '-1', '2': '-2'})
 
-    def test_ghashtable_utf8_container_in(self):
-        GIMarshallingTests.ghashtable_utf8_container_in({'-1': '1', '0': '0', '1': '-1', '2': '-2'})
-
-    def test_ghashtable_utf8_full_in(self):
-        GIMarshallingTests.ghashtable_utf8_full_in({'-1': '1', '0': '0', '1': '-1', '2': '-2'})
-
     def test_ghashtable_utf8_none_out(self):
         self.assertEquals({'-1': '1', '0': '0', '1': '-1', '2': '-2'}, GIMarshallingTests.ghashtable_utf8_none_out())
 
@@ -940,7 +925,6 @@ class TestGValue(unittest.TestCase):
 
     def test_gvalue_in(self):
         GIMarshallingTests.gvalue_in(42)
-        self.assertRaises(TypeError, GIMarshallingTests.gvalue_in, None)
 
     def test_gvalue_out(self):
         self.assertEquals(42, GIMarshallingTests.gvalue_out())
@@ -974,8 +958,9 @@ class TestEnum(unittest.TestCase):
 
     def test_enum_in(self):
         GIMarshallingTests.enum_in(GIMarshallingTests.Enum.VALUE3)
+        GIMarshallingTests.enum_in(42)
 
-        self.assertRaises(TypeError, GIMarshallingTests.enum_in, 42)
+        self.assertRaises(TypeError, GIMarshallingTests.enum_in, 43)
         self.assertRaises(TypeError, GIMarshallingTests.enum_in, 'GIMarshallingTests.Enum.VALUE3')
 
     def test_enum_out(self):
@@ -988,6 +973,16 @@ class TestEnum(unittest.TestCase):
         self.assertTrue(isinstance(enum, GIMarshallingTests.Enum))
         self.assertEquals(enum, GIMarshallingTests.Enum.VALUE1)
 
+    def test_enum_second(self):
+        # check for the bug where different non-gtype enums share the same class
+        self.assertNotEqual(GIMarshallingTests.Enum, GIMarshallingTests.SecondEnum)
+
+        # check that values are not being shared between different enums
+        self.assertTrue(hasattr(GIMarshallingTests.SecondEnum, "SECONDVALUE1"))
+        self.assertRaises(AttributeError, getattr, GIMarshallingTests.Enum, "SECONDVALUE1")
+        self.assertTrue(hasattr(GIMarshallingTests.Enum, "VALUE1"))
+        self.assertRaises(AttributeError, getattr, GIMarshallingTests.SecondEnum, "VALUE1")
+
 
 class TestGEnum(unittest.TestCase):
 
@@ -1000,8 +995,9 @@ class TestGEnum(unittest.TestCase):
 
     def test_genum_in(self):
         GIMarshallingTests.genum_in(GIMarshallingTests.GEnum.VALUE3)
+        GIMarshallingTests.genum_in(42)
 
-        self.assertRaises(TypeError, GIMarshallingTests.genum_in, 42)
+        self.assertRaises(TypeError, GIMarshallingTests.genum_in, 43)
         self.assertRaises(TypeError, GIMarshallingTests.genum_in, 'GIMarshallingTests.GEnum.VALUE3')
 
     def test_genum_out(self):
@@ -1076,7 +1072,7 @@ class TestStructure(unittest.TestCase):
         self.assertEquals(None, struct.pointer)
 
     def test_simple_struct_return(self):
-        struct = GIMarshallingTests.simple_struct_return()
+        struct = GIMarshallingTests.simple_struct_returnv()
 
         self.assertTrue(isinstance(struct, GIMarshallingTests.SimpleStruct))
         self.assertEquals(6, struct.long_)
@@ -1089,40 +1085,17 @@ class TestStructure(unittest.TestCase):
         struct.long_ = 6
         struct.int8 = 7
 
-        GIMarshallingTests.simple_struct_in(struct)
+        GIMarshallingTests.SimpleStruct.inv(struct)
 
         del struct
 
         struct = GIMarshallingTests.NestedStruct()
 
-        self.assertRaises(TypeError, GIMarshallingTests.simple_struct_in, struct)
+        self.assertRaises(TypeError, GIMarshallingTests.SimpleStruct.inv, struct)
 
         del struct
 
-        self.assertRaises(TypeError, GIMarshallingTests.simple_struct_in, None)
-
-    def test_simple_struct_out(self):
-        struct = GIMarshallingTests.simple_struct_out()
-
-        self.assertTrue(isinstance(struct, GIMarshallingTests.SimpleStruct))
-        self.assertEquals(6, struct.long_)
-        self.assertEquals(7, struct.int8)
-
-        del struct
-
-    def test_simple_struct_inout(self):
-        in_struct = GIMarshallingTests.SimpleStruct()
-        in_struct.long_ = 6
-        in_struct.int8 = 7
-
-        out_struct = GIMarshallingTests.simple_struct_inout(in_struct)
-
-        self.assertTrue(isinstance(out_struct, GIMarshallingTests.SimpleStruct))
-        self.assertEquals(7, out_struct.long_)
-        self.assertEquals(6, out_struct.int8)
-
-        del in_struct
-        del out_struct
+        self.assertRaises(TypeError, GIMarshallingTests.SimpleStruct.inv, None)
 
     def test_simple_struct_method(self):
         struct = GIMarshallingTests.SimpleStruct()
@@ -1145,7 +1118,7 @@ class TestStructure(unittest.TestCase):
         del struct
 
     def test_pointer_struct_return(self):
-        struct = GIMarshallingTests.pointer_struct_return()
+        struct = GIMarshallingTests.pointer_struct_returnv()
 
         self.assertTrue(isinstance(struct, GIMarshallingTests.PointerStruct))
         self.assertEquals(42, struct.long_)
@@ -1156,30 +1129,10 @@ class TestStructure(unittest.TestCase):
         struct = GIMarshallingTests.PointerStruct()
         struct.long_ = 42
 
-        GIMarshallingTests.pointer_struct_in(struct)
+        struct.inv()
 
         del struct
 
-    def test_pointer_struct_out(self):
-        struct = GIMarshallingTests.pointer_struct_out()
-
-        self.assertTrue(isinstance(struct, GIMarshallingTests.PointerStruct))
-        self.assertEquals(42, struct.long_)
-
-        del struct
-
-    def test_pointer_struct_inout(self):
-        in_struct = GIMarshallingTests.PointerStruct()
-        in_struct.long_ = 42
-
-        out_struct = GIMarshallingTests.pointer_struct_inout(in_struct)
-
-        self.assertTrue(isinstance(out_struct, GIMarshallingTests.PointerStruct))
-        self.assertEquals(0, out_struct.long_)
-
-        del in_struct
-        del out_struct
-
     def test_boxed_struct(self):
         self.assertTrue(issubclass(GIMarshallingTests.BoxedStruct, GObject.GBoxed))
 
@@ -1207,7 +1160,7 @@ class TestStructure(unittest.TestCase):
         del struct
 
     def test_boxed_struct_return(self):
-        struct = GIMarshallingTests.boxed_struct_return()
+        struct = GIMarshallingTests.boxed_struct_returnv()
 
         self.assertTrue(isinstance(struct, GIMarshallingTests.BoxedStruct))
         self.assertEquals(42, struct.long_)
@@ -1219,7 +1172,7 @@ class TestStructure(unittest.TestCase):
         struct = GIMarshallingTests.BoxedStruct()
         struct.long_ = 42
 
-        GIMarshallingTests.boxed_struct_in(struct)
+        struct.inv()
 
         del struct
 
@@ -1255,7 +1208,7 @@ class TestStructure(unittest.TestCase):
         del new_union
 
     def test_union_return(self):
-        union = GIMarshallingTests.union_return()
+        union = GIMarshallingTests.union_returnv()
 
         self.assertTrue(isinstance(union, GIMarshallingTests.Union))
         self.assertEquals(42, union.long_)
@@ -1266,30 +1219,10 @@ class TestStructure(unittest.TestCase):
         union = GIMarshallingTests.Union()
         union.long_ = 42
 
-        GIMarshallingTests.union_in(union)
+        union.inv()
 
         del union
 
-    def test_union_out(self):
-        union = GIMarshallingTests.union_out()
-
-        self.assertTrue(isinstance(union, GIMarshallingTests.Union))
-        self.assertEquals(42, union.long_)
-
-        del union
-
-    def test_union_inout(self):
-        in_union = GIMarshallingTests.Union()
-        in_union.long_ = 42
-
-        out_union = GIMarshallingTests.union_inout(in_union)
-
-        self.assertTrue(isinstance(out_union, GIMarshallingTests.Union))
-        self.assertEquals(0, out_union.long_)
-
-        del in_union
-        del out_union
-
     def test_union_method(self):
         union = GIMarshallingTests.Union()
         union.long_ = 42
@@ -1365,49 +1298,44 @@ class TestGObject(unittest.TestCase):
 #        self.assertEquals(object_.int_, 42)
 
     def test_object_none_return(self):
-        object_ = GIMarshallingTests.object_none_return()
+        object_ = GIMarshallingTests.Object.none_return()
         self.assertTrue(isinstance(object_, GIMarshallingTests.Object))
         self.assertEquals(object_.__grefcount__, 2)
 
     def test_object_full_return(self):
-        object_ = GIMarshallingTests.object_full_return()
+        object_ = GIMarshallingTests.Object.full_return()
         self.assertTrue(isinstance(object_, GIMarshallingTests.Object))
         self.assertEquals(object_.__grefcount__, 1)
 
     def test_object_none_in(self):
         object_ = GIMarshallingTests.Object(int = 42)
-        GIMarshallingTests.object_none_in(object_)
+        GIMarshallingTests.Object.none_in(object_)
         self.assertEquals(object_.__grefcount__, 1)
 
         object_ = GIMarshallingTests.SubObject(int = 42)
-        GIMarshallingTests.object_none_in(object_)
+        GIMarshallingTests.Object.none_in(object_)
 
         object_ = GObject.GObject()
-        self.assertRaises(TypeError, GIMarshallingTests.object_none_in, object_)
+        self.assertRaises(TypeError, GIMarshallingTests.Object.none_in, object_)
 
-        self.assertRaises(TypeError, GIMarshallingTests.object_none_in, None)
-
-    def test_object_full_in(self):
-        object_ = GIMarshallingTests.Object(int = 42)
-        GIMarshallingTests.object_full_in(object_)
-        self.assertEquals(object_.__grefcount__, 1)
+        self.assertRaises(TypeError, GIMarshallingTests.Object.none_in, None)
 
     def test_object_none_out(self):
-        object_ = GIMarshallingTests.object_none_out()
+        object_ = GIMarshallingTests.Object.none_out()
         self.assertTrue(isinstance(object_, GIMarshallingTests.Object))
         self.assertEquals(object_.__grefcount__, 2)
 
-        new_object = GIMarshallingTests.object_none_out()
+        new_object = GIMarshallingTests.Object.none_out()
         self.assertTrue(new_object is object_)
 
     def test_object_full_out(self):
-        object_ = GIMarshallingTests.object_full_out()
+        object_ = GIMarshallingTests.Object.full_out()
         self.assertTrue(isinstance(object_, GIMarshallingTests.Object))
         self.assertEquals(object_.__grefcount__, 1)
 
     def test_object_none_inout(self):
         object_ = GIMarshallingTests.Object(int = 42)
-        new_object = GIMarshallingTests.object_none_inout(object_)
+        new_object = GIMarshallingTests.Object.none_inout(object_)
 
         self.assertTrue(isinstance(new_object, GIMarshallingTests.Object))
 
@@ -1416,20 +1344,20 @@ class TestGObject(unittest.TestCase):
         self.assertEquals(object_.__grefcount__, 1)
         self.assertEquals(new_object.__grefcount__, 2)
 
-        new_new_object = GIMarshallingTests.object_none_inout(object_)
+        new_new_object = GIMarshallingTests.Object.none_inout(object_)
         self.assertTrue(new_new_object is new_object)
 
-        GIMarshallingTests.object_none_inout(GIMarshallingTests.SubObject(int = 42))
+        GIMarshallingTests.Object.none_inout(GIMarshallingTests.SubObject(int = 42))
 
     def test_object_full_inout(self):
         object_ = GIMarshallingTests.Object(int = 42)
-        new_object = GIMarshallingTests.object_full_inout(object_)
+        new_object = GIMarshallingTests.Object.full_inout(object_)
 
         self.assertTrue(isinstance(new_object, GIMarshallingTests.Object))
 
         self.assertFalse(object_ is new_object)
 
-        self.assertEquals(object_.__grefcount__, 1)
+        self.assertEquals(object_.__grefcount__, 2)
         self.assertEquals(new_object.__grefcount__, 1)
 
 # FIXME: Doesn't actually return the same object.
@@ -1443,8 +1371,6 @@ class TestGObject(unittest.TestCase):
 class TestPythonGObject(unittest.TestCase):
 
     class Object(GIMarshallingTests.Object):
-        __gtype_name__ = "Object"
-
         def __init__(self, int):
             GIMarshallingTests.Object.__init__(self)
             self.val = None
@@ -1460,7 +1386,16 @@ class TestPythonGObject(unittest.TestCase):
             return 42
 
         def do_method_with_default_implementation(self, int8):
-            self.props.int = int8 * 2
+            GIMarshallingTests.Object.do_method_with_default_implementation(self, int8)
+            self.props.int += int8
+
+    class SubObject(GIMarshallingTests.SubObject):
+        def __init__(self, int):
+            GIMarshallingTests.SubObject.__init__(self)
+            self.val = None
+
+        def do_method_with_default_implementation(self, int8):
+            self.val = int8
 
     def test_object(self):
         self.assertTrue(issubclass(self.Object, GIMarshallingTests.Object))
@@ -1478,11 +1413,9 @@ class TestPythonGObject(unittest.TestCase):
         self.assertEqual(object_.method_int8_out(), 42)
 
         object_.method_with_default_implementation(42)
-        self.assertEqual(object_.val, 84)
+        self.assertEqual(object_.props.int, 84)
 
         class ObjectWithoutVFunc(GIMarshallingTests.Object):
-            __gtype_name__ = 'ObjectWithoutVFunc'
-
             def __init__(self, int):
                 GIMarshallingTests.Object.__init__(self)
 
@@ -1490,6 +1423,11 @@ class TestPythonGObject(unittest.TestCase):
         object_.method_with_default_implementation(84)
         self.assertEqual(object_.props.int, 84)
 
+    def test_subobject_parent_vfunc(self):
+        object_ = self.SubObject(int = 81)
+        object_.method_with_default_implementation(87)
+        self.assertEquals(object_.val, 87)
+
     def test_dynamic_module(self):
         from gi.module import DynamicGObjectModule
         self.assertTrue(isinstance(GObject, DynamicGObjectModule))
@@ -1498,6 +1436,44 @@ class TestPythonGObject(unittest.TestCase):
         # compare a static gobject attr with a dynamic GObject attr
         self.assertEquals(GObject.GObject, gobject.GObject)
 
+    def test_subobject_non_vfunc_do_method(self):
+        class PythonObjectWithNonVFuncDoMethod:
+            def do_not_a_vfunc(self):
+                return 5
+
+        class ObjectOverrideNonVFuncDoMethod(GIMarshallingTests.Object, PythonObjectWithNonVFuncDoMethod):
+            def do_not_a_vfunc(self):
+                value = super(ObjectOverrideNonVFuncDoMethod, self).do_not_a_vfunc()
+                return 13 + value
+
+        object_ = ObjectOverrideNonVFuncDoMethod()
+        self.assertEquals(18, object_.do_not_a_vfunc())
+
+    def test_native_function_not_set_in_subclass_dict(self):
+        # Previously, GI was setting virtual functions on the class as well
+        # as any *native* class that subclasses it. Here we check that it is only
+        # set on the class that the method is originally from.
+        self.assertTrue('do_method_with_default_implementation' in GIMarshallingTests.Object.__dict__)
+        self.assertTrue('do_method_with_default_implementation' not in GIMarshallingTests.SubObject.__dict__)
+
+        # Here we check that accessing a vfunc from the subclass returns the same wrapper object,
+        # meaning that multiple wrapper objects have not been created for the same vfunc.
+        func1 = GIMarshallingTests.Object.do_method_with_default_implementation
+        func2 = GIMarshallingTests.SubObject.do_method_with_default_implementation
+        if sys.version_info < (3,0):
+            func1 = func1.im_func
+            func2 = func2.im_func
+            
+        self.assertTrue(func1 is func2)
+
+    def test_subobject_with_interface_and_non_vfunc_do_method(self):
+        # There was a bug for searching for vfuncs in interfaces. It was
+        # triggered by having a do_* method that wasn't overriding
+        # a native vfunc, as well as inheriting from an interface.
+        class GObjectSubclassWithInterface(GObject.GObject, GIMarshallingTests.Interface):
+            def do_method_not_a_vfunc(self):
+                pass
+
 class TestMultiOutputArgs(unittest.TestCase):
 
     def test_int_out_out(self):
@@ -1506,6 +1482,17 @@ class TestMultiOutputArgs(unittest.TestCase):
     def test_int_return_out(self):
         self.assertEquals((6, 7), GIMarshallingTests.int_return_out())
 
+class TestGErrorException(unittest.TestCase):
+    def test_gerror_exception(self):
+        self.assertRaises(GObject.GError, GIMarshallingTests.gerror)
+        try:
+            GIMarshallingTests.gerror()
+        except Exception:
+            etype, e = sys.exc_info()[:2]
+            self.assertEquals(e.domain, GIMarshallingTests.CONSTANT_GERROR_DOMAIN)
+            self.assertEquals(e.code, GIMarshallingTests.CONSTANT_GERROR_CODE)
+            self.assertEquals(e.message, GIMarshallingTests.CONSTANT_GERROR_MESSAGE)
+
 
 # Interface
 
@@ -1519,7 +1506,6 @@ class TestInterfaces(unittest.TestCase):
     def test_implementation(self):
 
         class TestInterfaceImpl(GObject.GObject, GIMarshallingTests.Interface):
-            __gtype_name__ = 'TestInterfaceImpl'
             def __init__(self):
                 GObject.GObject.__init__(self)
                 self.val = None
@@ -1536,24 +1522,26 @@ class TestInterfaces(unittest.TestCase):
         self.assertEquals(instance.val, 42)
 
         class TestInterfaceImplA(TestInterfaceImpl):
-            __gtype_name__ = 'TestInterfaceImplA'
+            pass
 
         class TestInterfaceImplB(TestInterfaceImplA):
-            __gtype_name__ = 'TestInterfaceImplB'
+            pass
 
         instance = TestInterfaceImplA()
         GIMarshallingTests.test_interface_test_int8_in(instance, 42)
         self.assertEquals(instance.val, 42)
 
-        def define_implementor_without_gtype():
-            class TestInterfaceImpl(gobject.GObject, GIMarshallingTests.Interface):
-                def __init__(self):
-                    gobject.GObject.__init__(self)
-                    self.val = None
 
+class TestInterfaceClash(unittest.TestCase):
+
+    def test_clash(self):
+        def create_clash():
+            class TestClash(GObject.GObject, GIMarshallingTests.Interface, GIMarshallingTests.Interface2):
                 def do_test_int8_in(self, int8):
-                    self.val = int8
-        self.assertRaises(RuntimeError, define_implementor_without_gtype)
+                    pass
+            TestClash()
+
+        self.assertRaises(TypeError, create_clash)
 
 
 class TestOverrides(unittest.TestCase):
@@ -1573,7 +1561,7 @@ class TestOverrides(unittest.TestCase):
         del struct
 
         # Test that the overrides wrapper has been registered.
-        struct = GIMarshallingTests.overrides_struct_return()
+        struct = GIMarshallingTests.overrides_struct_returnv()
 
         self.assertTrue(isinstance(struct, GIMarshallingTests.OverridesStruct))
 
@@ -1594,10 +1582,34 @@ class TestOverrides(unittest.TestCase):
         self.assertEquals(6, object_.method())
 
         # Test that the overrides wrapper has been registered.
-        object_ = GIMarshallingTests.overrides_object_return()
+        object_ = GIMarshallingTests.OverridesObject.returnv()
 
         self.assertTrue(isinstance(object_, GIMarshallingTests.OverridesObject))
 
     def test_module_name(self):
         self.assertEquals(GIMarshallingTests.OverridesStruct.__module__, 'gi.overrides.GIMarshallingTests')
         self.assertEquals(GObject.InitiallyUnowned.__module__, 'gi.repository.GObject')
+
+class TestDir(unittest.TestCase):
+    def test_members_list(self):
+        list = dir(GIMarshallingTests)
+        self.assertTrue('OverridesStruct' in list)
+        self.assertTrue('BoxedStruct' in list)
+        self.assertTrue('OVERRIDES_CONSTANT' in list)
+        self.assertTrue('GEnum' in list)
+        self.assertTrue('int32_return_max' in list)
+
+    def test_modules_list(self):
+        import gi.repository
+        list = dir(gi.repository)
+        self.assertTrue('GIMarshallingTests' in list)
+
+        # FIXME: test to see if a module which was not imported is in the list
+        #        we should be listing every typelib we find, not just the ones
+        #        which are imported
+        #
+        #        to test this I recommend we compile a fake module which
+        #        our tests would never import and check to see if it is
+        #        in the list:
+        #
+        # self.assertTrue('DoNotImportDummyTests' in list)
